-------------------------------------------------------------------------------
--! @file       Keccak_F_Datapath.vhd
--! @brief      Datapath of Keccak_F
--! @project    CAESAR Candidate Evaluation  
--! @author     Ekawat (ice) Homsirikamol
--! @version    1.0
--! @copyright  Copyright (c) 2014 Cryptographic Engineering Research Group
--!             ECE Department, George Mason University Fairfax, VA, U.S.A.
--!             All rights Reserved.
--! @license    This project is released under the GNU Public License.
--!             The license and distribution terms for this file may be
--!             found in the file LICENSE in this distribution or at 
--!             http://www.gnu.org/licenses/gpl-3.0.txt
--! @note       This is publicly available encryption source code that falls
--!             under the License Exception TSU (Technology and software-
--!             —unrestricted)
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.Keccak_pkg.all;

entity Keccak_F_Datapath is    
    port (
        --! Global signals
        clk             : in  std_logic;
        rst             : in  std_logic;

        --! Data signals
        din             : in  t_Keccak_state;
        dout            : out t_Keccak_state;
        
        --! Control signals
        round           : in  std_logic_vector(4 downto 0);
        sel_in          : in  std_logic;
        en_in           : in  std_logic
    );
end Keccak_F_Datapath;

-------------------------------------------------------------------------------
--! @brief  Architecture definition of Keccak_F_Datapath
-------------------------------------------------------------------------------
architecture structure of Keccak_F_Datapath is        
    signal from_register     : t_Keccak_state;
    signal from_round        : t_Keccak_state;    
    signal rc                : std_logic_vector(63   downto 0);

    type type_rom is array (0 to 31) of std_logic_vector(63 downto 0);
    constant my_rom: type_rom :=(
        x"0000000000000001", x"0000000000008082", x"800000000000808A", x"8000000080008000",
        x"000000000000808B", x"0000000080000001", x"8000000080008081", x"8000000000008009",
        x"000000000000008A", x"0000000000000088", x"0000000080008009", x"000000008000000A",
        x"000000008000808B", x"800000000000008B", x"8000000000008089", x"8000000000008003",
        x"8000000000008002", x"8000000000000080", x"000000000000800A", x"800000008000000A",
        x"8000000080008081", x"8000000000008080", x"0000000080000001", x"8000000080008008",
        x"0000000000000000", x"0000000000000000", x"0000000000000000", x"0000000000000000",
        x"0000000000000000", x"0000000000000000", x"0000000000000000", x"0000000000000000");
begin
    
    preg:
    process( clk )
    begin
        if rising_edge( clk ) then
            if en_in = '1' then
                if sel_in = '1' then
                    from_register <= din;
                else
                    from_register <= from_round;
                end if;
                rc <= my_rom(to_integer(unsigned(round)));
            end if;
        end if;
    end process;

    rd      : entity work.Keccak_Round(basic) port map (rc=>rc, rin=>from_register, rout=>from_round);
    
    dout <= from_round;
end structure;