-------------------------------------------------------------------------------
--! @file       Keccak_invmap.vhd
--! @brief      Inverse mapping of Keccak state to std_logic
--!
--! This unit converts from t_Keccak_state to std_logic_vector.
--!
--! order of state words on a bus (most-significant to least-significant position)
--! (0,0), (1,0), (2,0), (3,0), (4,0),
--! (0,1), (1,1), (2,1), (3,1), (4,1),
--! (0,2), (1,2), (2,2), (3,2), (4,2),
--! (0,3), (1,3), (2,3), (3,3), (4,3),
--! (0,4), (1,4), (2,4), (3,4), (4,4)
--! order of state bits in state words (most-significant to least-significant position)
--! (0..KECCAK_WORD_SIZE-1)
--!
--! @project    CAESAR Candidate Evaluation  
--! @author     Ekawat (ice) Homsirikamol
--! @version    1.0
--! @copyright  Copyright (c) 2014 Cryptographic Engineering Research Group
--!             ECE Department, George Mason University Fairfax, VA, U.S.A.
--!             All rights Reserved.
--! @license    This project is released under the GNU Public License.
--!             The license and distribution terms for this file may be
--!             found in the file LICENSE in this distribution or at 
--!             http://www.gnu.org/licenses/gpl-3.0.txt
--! @note       This is publicly available encryption source code that falls
--!             under the License Exception TSU (Technology and software-
--!             —unrestricted)
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

use work.Keccak_pkg.all;

entity Keccak_invmap is  
    port (
        ii :   in  t_Keccak_state;
        oo :   out std_logic_vector(KECCAK_b-1 downto 0)
    );
end Keccak_invmap;

-------------------------------------------------------------------------------
--! @brief  Architecture definition of Keccak_invmap
-------------------------------------------------------------------------------
architecture structure of Keccak_invmap is   
begin
    gRow: for i in 0 to 4 generate
        gCol: for j in 0 to 4 generate
            oo(KECCAK_b-(5*i+j)*KECCAK_WORD_SIZE-1 downto KECCAK_b-(5*i+j)*KECCAK_WORD_SIZE-KECCAK_WORD_SIZE)  <=  ii(j,i);
        end generate;
    end generate;
end structure;