-------------------------------------------------------------------------------
--! @file       Keccak_pkg.vhd
--! @brief      Package file used by Keccak_F and its sub-modules
--! @project    CAESAR Candidate Evaluation
--! @author     Ekawat (ice) Homsirikamol
--! @version    1.0
--! @copyright  Copyright (c) 2014 Cryptographic Engineering Research Group
--!             ECE Department, George Mason University Fairfax, VA, U.S.A.
--!             All rights Reserved.
--! @license    This project is released under the GNU Public License.
--!             The license and distribution terms for this file may be
--!             found in the file LICENSE in this distribution or at 
--!             http://www.gnu.org/licenses/gpl-3.0.txt
--! @note       This is publicly available encryption source code that falls
--!             under the License Exception TSU (Technology and software-
--!             —unrestricted)
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

package Keccak_pkg is
    constant KECCAK_b           : integer := 1600;
    constant KECCAK_WORD_SIZE   : integer := KECCAK_b/25;
    type t_Keccak_state is array (0 to 4, 0 to 4) of std_logic_vector(KECCAK_WORD_SIZE-1 downto 0);
    type t_Keccak_sheet is array (0 to 4)         of std_logic_vector(KECCAK_WORD_SIZE-1 downto 0);
    type t_Rotate       is array (0 to 4, 0 to 4) of integer;
   constant rotateOffset : t_Rotate := (( 0,36, 3,41,18),
                                        ( 1,44,10,45, 2),
                                        (62, 6,43,15,61),
                                        (28,55,25,21,56),
                                        (27,20,39, 8,14));
end Keccak_pkg;